<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Http\Request;
use Laravelista\Algo\Models\Post;
use Laravelista\Algo\Models\Image;
use Illuminate\Support\Facades\Storage;
use Laravelista\Picasso\Picasso;

class PostImageController extends Controller
{
    protected $picasso;

    public function __construct(Picasso $picasso)
    {
        $this->picasso = $picasso;
    }

    /**
     * Display a listing of the resource.
     *
     * @param  Post  $post
     * @return \Illuminate\Http\Response
     */
    public function index(Post $post)
    {
        $images = Image::where('post_id', $post->id)->latest()->get();

        return view('algo::posts.images.index')
            ->withPost($post)
            ->withImages($images);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param  Post  $post
     * @return \Illuminate\Http\Response
     */
    public function create(Post $post)
    {
        return view('algo::posts.images.create')->withPost($post);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  Post  $post
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, Post $post)
    {
        $request->validate([
            'image' => ['required', 'file', 'mimes:jpeg,jpg,png'],
        ]);

        $image = new Image;
        $image->path = $request->image->store('images', 'originals');
        $image->post()->associate($post->id);
        $image->save();

        $this->picasso->optimize($image->path, config('algo.post_dimensions'), 'public');

        return redirect()
            ->route('algo.posts.images.index', $post->id)
            ->with('status', 'Slika napravljena!');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  Post  $post
     * @param  Image  $image
     * @return \Illuminate\Http\Response
     */
    public function edit(Post $post, Image $image)
    {
        return view('algo::posts.images.edit')
            ->withPost($post)
            ->withImage($image);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  Post  $post
     * @param  Image  $image
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Post $post, Image $image, Storage $storage)
    {
        $request->validate([
            'image' => ['required', 'file', 'mimes:jpeg,jpg,png'],
        ]);

        $storage::disk('originals')->delete($image->path);
        $this->picasso->purge($image->path, 'public');

        $image->path = $request->image->store('images', 'originals');
        $image->save();

        $this->picasso->optimize($image->path, config('algo.post_dimensions'), 'public');

        return redirect()
            ->route('algo.posts.images.index', $post->id)
            ->with('status', 'Slika uređena!');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  Post  $post
     * @param  Image  $image
     * @return \Illuminate\Http\Response
     */
    public function destroy(Post $post, Image $image, Storage $storage)
    {
        $storage::disk('originals')->delete($image->path);
        $this->picasso->purge($image->path, 'public');

        $image->delete();

        return redirect()
            ->route('algo.posts.images.index', $post)
            ->with('status', 'Slika obrisana!');
    }
}
