<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Storage;
use Laravelista\Algo\Models\Tag;
use Illuminate\Validation\Rule;
use Laravelista\Picasso\Picasso;
use Laravelista\Algo\Helpers\Slug;

class TagController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $tags = Tag::with(['translations'])->get()->sortBy('name');

        return view('algo::tags.index')->withTags($tags);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('algo::tags.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param Slug $slug
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, Slug $slug, Picasso $picasso)
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('tag_translations')->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $tag = new Tag;
        $tag->translateOrNew($locale)->name = $validatedData['name'];
        $tag->translateOrNew($locale)->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $tag->translateOrNew($locale)->description = $validatedData['description'];
        if ($request->hasFile('image')) {
            $tag->image = $request->image->store('images', 'originals');
        }
        $tag->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($tag->image, config('algo.tag_dimensions'), 'public');
        }

        return redirect()->route('algo.tags.index')->with('status', 'Oznaka napravljena!');
    }

    /**
     * Display the specified resource.
     *
     * @param \Laravelista\Algo\Tag  $tag
     * @return \Illuminate\Http\Response
     */
    public function show(Tag $tag)
    {
        return view('algo::tags.show')->withTag($tag);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \Laravelista\Algo\Tag  $tag
     * @return \Illuminate\Http\Response
     */
    public function edit(Tag $tag)
    {
        return view('algo::tags.edit')->withTag($tag);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param \Laravelista\Algo\Tag  $tag
     * @param  Slug $slug
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Tag $tag, Slug $slug, Picasso $picasso, Storage $storage)
    {
        $locale = App::getLocale();

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique('tag_translations')
                    ->ignore($tag->id, 'tag_id')
                    ->where('locale', $locale)
            ],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'description' => ['nullable', 'string'],
        ]);

        $tag->translateOrNew($locale)->name = $validatedData['name'];
        $tag->translateOrNew($locale)->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $tag->translateOrNew($locale)->description = $validatedData['description'];

        if ($request->hasFile('image') and $tag->hasImage()) {
            $storage::disk('originals')->delete($tag->image);
            $picasso->purge($tag->image, 'public');
        }

        if ($request->hasFile('image')) {
            $tag->image = $request->image->store('images', 'originals');
        }

        $tag->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($tag->image, config('algo.tag_dimensions'), 'public');
        }

        return redirect()
            ->route('algo.tags.index')
            ->with('status', 'Oznaka uređena!');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \Laravelista\Algo\Tag  $tag
     * @return \Illuminate\Http\Response
     */
    public function destroy(Tag $tag, Storage $storage, Picasso $picasso)
    {
        if ($tag->hasImage()) {
            $storage::disk('originals')->delete($tag->image);
            $picasso->purge($tag->image, 'public');
        }

        $tag->delete();

        return redirect()
            ->route('algo.tags.index')
            ->with('status', 'Oznaka obrisana!');
    }
}
