<?php

namespace Laravelista\Algo\Models;

use Illuminate\Database\Eloquent\Model;
use Astrotomic\Translatable\Translatable;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Str;

class Post extends Model
{
    use Translatable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = ['published_at', 'content', 'intro', 'title', 'video_link', 'slug', 'featured'];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'featured' => 'boolean'
    ];

    /**
     * The attributes that are translatable.
     *
     * @var array
     */
    public $translatedAttributes = ['content', 'intro', 'title', 'slug'];

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = ['published_at'];

    /**
     * It returns the author of the post.
     */
    public function author()
    {
        return $this->belongsTo(Config::get('algo.user'));
    }

    /**
     * It returns the tags for this post.
     */
    public function tags()
    {
        return $this->belongsToMany(Tag::class);
    }

    /**
     * It returns the category to which this post belongs to.
     */
    public function category()
    {
        return $this->belongsTo(Category::class);
    }

    /**
     * It returns images for this post.
     */
    public function images()
    {
        return $this->hasMany(Image::class);
    }

    /**
     * It returns the excerpt or content limited to chars.
     *
     * @param integer $chars
     * @return string
     */
    public function getExcerpt($chars = 200)
    {
        return $this->intro ?? Str::limit(strip_tags((new \Parsedown)->parse($this->content)), $chars);
    }

    /**
     * It converts content from markdown to html
     * and returns it as text.
     *
     * @return string
     */
    public function getContent()
    {
        return (new \Parsedown)->parse($this->content);
    }

    /**
     * It returns the reading time in minutes.
     */
    public function getReadingTime($wordsPerMinute = 200): int
    {
        $words = str_word_count($this->content);

        return (int) ceil($words / $wordsPerMinute);
    }

    /**
     * It returns only currently published posts, not scheduled posts.
     */
    public function scopePublished($query)
    {
        return $query->whereNotNull('published_at')
            ->where('published_at', '<=', Carbon::now());
    }

    /**
     * It only check if the published_at is null or
     * if the published_at is less than or equals to now.
     *
     * @return boolean
     */
    public function isPublished()
    {
        if (is_null($this->published_at)) {
            return false;
        }

        if ($this->published_at->lte(Carbon::now())) {
            return true;
        }

        return false;
    }

    /**
     * It returns true if the post is featured,
     * or false if it is not featured.
     *
     * @return boolean
     */
    public function isFeatured()
    {
        return $this->featured;
    }

    /**
     * It returns only posts that are featured.
     */
    public function scopeFeatured($query)
    {
        return $query->where('featured', true);
    }

    /**
     * It returns the video link in correct youtube embed url.
     *
     * @return string
     */
    public function getYouTubeEmbedLink()
    {
        return str_replace('watch?v=', 'embed/', $this->video_link);
    }

    /**
     * Depending on the post attributes it returns
     * the correct post format to display it in.
     */
    public function getPostFormat()
    {
        if (!empty($this->video_link) and $this->images->count() > 0) {
            return 'video';
        }

        if ($this->images->count() > 1) {
            return 'gallery';
        }

        if ($this->images->count() == 1) {
            return 'image';
        }

        return 'no-image';
    }
}
